@@ -17,6 +17,8 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include <Python.h>
+#include "cfg.h"
 #include "cards.h"
 
 /* Return the name of a card */
@@ -110,6 +112,66 @@ gchar*	card_get_name(Card *card)
 	g_assert_not_reached();
 }
 
+/* Get the point value of a linked list of cards */
+gint cards_get_points(GList *list)
+{
+	GList *l;
+	
+	/* Figure out how many point cards are in the list */
+	gint point_cards = 0;
+	for (l = list; l; l = l->next)
+	{
+		Card* card = l->data;
+		
+		if (card->suit == CARDS_HEARTS || (card->suit == CARDS_SPADES && card->rank == CARD_QUEEN))
+			point_cards ++;
+	}
+	
+	/* Shooting the moon: substract points for hearts and the Queen of spades */
+	gint modifier = point_cards == 14 ? -1 : 1;
+	
+	/* Shooting the sun: substract double points for hearts and the Queen of spades */
+	if (g_list_length(list) == 52)
+		modifier = -2;
+	
+	gint score = 0;
+	for (l = list; l; l = l->next)
+	{
+		Card* card = l->data;
+		
+		/* Hearts add points */
+		if (card->suit == CARDS_HEARTS)
+		{
+			if (cfg.ruleset == RULESET_SPOT_HEARTS)
+				/* variant: Spot hearts - card is worth their rank */
+				score += card->rank * modifier;
+			else
+				score += modifier;
+		}
+		
+		/* The Queen of spades adds a big penalty */
+		else if (card->suit == CARDS_SPADES && card->rank == CARD_QUEEN)
+		{
+			if (cfg.ruleset == RULESET_SPOT_HEARTS)
+				/* variant: Spot hearts - Queen of spades is 25 points */
+				score += 25 * modifier;
+			else
+				score += 13 * modifier;
+		}
+		
+		/* variant: Omnibus - Jack of diamonds is -10 points */
+		else if (cfg.ruleset == RULESET_OMNIBUS && card->suit == CARDS_DIAMONDS && card->rank == CARD_JACK)
+			score -= 10;
+		
+		/* variant: Alternative Omnibus - Ten of diamonds is -10 points */
+		else if (cfg.ruleset == RULESET_OMNIBUS_ALT && card->suit == CARDS_DIAMONDS && card->rank == CARD_TEN)
+			score -= 10;
+	}
+	
+	return score;
+}
+
+
 /* create a new deck of cards */
 CardsDeck* cards_deck_new (gboolean aces_high, gboolean include_jokers)
 {
@@ -232,7 +294,7 @@ void cards_deck_free (CardsDeck *deck)
 }
 
 /* create an empty hand */
-CardsHand* cards_hand_new (gint direction, gint orientation)
+CardsHand* cards_hand_new (gint direction, gboolean show_faces)
 {
 	CardsHand* hand;
 	
@@ -240,7 +302,7 @@ CardsHand* cards_hand_new (gint direction, gint orientation)
 	hand->list = NULL;
 	hand->hist = NULL;
 	hand->direction = direction;
-	hand->orientation = orientation;
+	hand->show_faces = show_faces;
 	
 	return hand;
 }
@@ -341,7 +403,7 @@ void cards_hand_get_area(CardsHand *hand, GtkWidget *widget, CardsImage *image,
 	gint offset_x = 0;
 	gint offset_y = 0;
 	
-	if (hand->orientation == NORTH || hand->orientation == SOUTH)
+	if (hand->direction == NORTH || hand->direction == SOUTH)
 	{
 		offset_x = image->width / 4;
 		
@@ -352,7 +414,7 @@ void cards_hand_get_area(CardsHand *hand, GtkWidget *widget, CardsImage *image,
 		*height = image->height + offset_y;
 		*x = (widget->allocation.width - *width) / 2;
 		
-		if (hand->orientation == NORTH)
+		if (hand->direction == NORTH)
 			*y = 10;
 		else
 			*y = widget->allocation.height - *height - 10;
@@ -368,7 +430,7 @@ void cards_hand_get_area(CardsHand *hand, GtkWidget *widget, CardsImage *image,
 		*width = image->width + offset_x;
 		*y = (widget->allocation.height - *height) / 2;
 		
-		if (hand->orientation == WEST)
+		if (hand->direction == WEST)
 			*x = 10;
 		else
 		{
@@ -390,7 +452,7 @@ void cards_hand_render(CardsHand *hand, CardsImage *cards, GdkPixmap *target, Gd
 	offset_x = 0;
 	offset_y = 0;
 	
-	switch (hand->orientation)
+	switch (hand->direction)
 	{
 		case NORTH:	offset_y = cards->height /  4; break;
 		case EAST:	offset_x = cards->width  / -4; x -= offset_x; break;
@@ -405,7 +467,7 @@ void cards_hand_render(CardsHand *hand, CardsImage *cards, GdkPixmap *target, Gd
 		card = (Card*)i->data;
 		
 		/* get the image ID */
-		if (hand->direction == FACE_UP)
+		if (hand->show_faces == FACE_UP)
 			id = card_id(card);
 		else
 			id = CARD_BACK;
@@ -417,7 +479,7 @@ void cards_hand_render(CardsHand *hand, CardsImage *cards, GdkPixmap *target, Gd
 			cards_image_render_to_pixmap(cards, id, card->active, target, target_gc, x, y);
 		
 		/* position of the next card */
-		if (hand->orientation == NORTH || hand->orientation == SOUTH)
+		if (hand->direction == NORTH || hand->direction == SOUTH)
 			x += (cards->width / 4);
 		else
 			y += (cards->height / 4);
@@ -504,47 +566,6 @@ gint cards_hand_num_selected(CardsHand *hand)
 	return num_selected;
 }
 
-/* publish a hand of cards in Lua as the global "hand" */
-void cards_hand_publish(CardsHand *hand, lua_State *state)
-{
-	GList *i;
-	Card *card;
-	gint index;
-	
-	/* start a new table */
-	lua_newtable(state);							/* table */
-	
-	/* add all the cards to the table */
-	i = hand->list;
-	index = 1;
-	while (i != NULL)
-	{
-		card = (Card*)i->data;
-		g_assert(card != NULL);
-		
-		/* push this card's index */
-		lua_pushnumber(state, index);				/* table, key */
-		
-		/* push the card value (another table) */
-		lua_newtable(state);						/* table, key, table */
-		lua_pushnumber(state, 1);					/* table, key, table, key */
-		lua_pushnumber(state, card->suit);			/* table, key, table, key, val */
-		lua_settable(state, -3);					/* table, key, table */
-		lua_pushnumber(state, 2);					/* table, key, table, key */
-		lua_pushnumber(state, card->rank);			/* table, key, table, key, val */
-		lua_settable(state, -3);					/* table, key, table */
-		
-		/* now add it to the table */
-		lua_settable(state, -3);					/* table */
-		
-		i = g_list_next(i);
-		index++;
-	}
-	
-	/* assign the table to the variable "hand" */
-	lua_setglobal(state, "hand");					/* <empty> */
-}
-
 /* reset the hand to the initial state */
 void cards_hand_reset(CardsHand *hand)
 {
@@ -599,39 +620,20 @@ gint trick_get_winner(Trick *trick)
 }
 
 /* get the score of a trick */
-gint trick_get_score(Trick *trick, gint ruleset)
+gint trick_get_score(Trick *trick)
 {
 	gint i, score = 0;
+	GList *list = NULL;
 	
+	/* cards_get_score wants a linked list, so build it */
 	for (i = 0; i < 4; i++)
 	{
-		if (trick->card[i] == NULL)
-			continue;
-		
-		if (trick->card[i]->suit == CARDS_HEARTS)
-		{
-			if (ruleset == RULESET_SPOT_HEARTS)
-				/* variant: Spot hearts - card is worth their rank */
-				score += trick->card[i]->rank;
-			else
-				score++;
-		}
-		else if (trick->card[i]->suit == CARDS_SPADES && trick->card[i]->rank == CARD_QUEEN)
-		{
-			if (ruleset == RULESET_SPOT_HEARTS)
-				/* variant: Spot hearts - Queen of spades is 25 points */
-				score += 25;
-			else
-				score += 13;
-		}
-		/* variant: Omnibus - Jack of diamonds is -10 points */
-		else if (ruleset == RULESET_OMNIBUS && trick->card[i]->suit == CARDS_DIAMONDS && trick->card[i]->rank == CARD_JACK)
-			score -= 10;
-		/* variant: Alternative Omnibus - Ten of diamonds is -10 points */
-		else if (ruleset == RULESET_OMNIBUS_ALT && trick->card[i]->suit == CARDS_DIAMONDS && trick->card[i]->rank == CARD_TEN)
-			score -= 10;
+		list = g_list_append(list, trick->card[i]);
 	}
 	
+	score = cards_get_points(list);
+	g_list_free(list);
+	
 	return score;
 }
 
@@ -720,41 +722,31 @@ void trick_reset(Trick *trick)
 	
 }
 
-/* publisk a trick in Lua as the global "trick" */
-void trick_publish(Trick *trick, lua_State *state)
+/* Set the trick global in Python */
+void trick_publish(Trick *trick)
 {
-	gint i;
+	PyObject *module, *dict, *object, *result, *card[4];
+	int i;
 	
-	/* start a new table */
-	lua_newtable(state);							/* table */
-
-	for (i = 0; i < 4; i ++)
+	/* Get a reference to trick.set() */
+	module = PyImport_AddModule("__main__");
+	dict = PyModule_GetDict(module);
+	object = PyDict_GetItemString(dict, "trick");
+	
+	/* Create a tuple of cards */
+	for (i = 0; i < 4; i++)
 	{
-		/* push the key */
-		lua_pushnumber(state, i + 1);				/* table, key */
-		
-		/* push the value */
 		if (trick->card[i] == NULL)
-		{
-			lua_pushnil(state);						/* table, key, nil */
-		}
+			card[i] = Py_BuildValue("");
 		else
-		{
-			lua_newtable(state);							/* table, key, table */
-			lua_pushnumber(state, 1);						/* table, key, table, key */
-			lua_pushnumber(state, trick->card[i]->suit);	/* table, key, table, key, val */
-			lua_settable(state, -3);						/* table, key, table */
-			lua_pushnumber(state, 2);						/* table, key, table, key */
-			lua_pushnumber(state, trick->card[i]->rank);	/* table, key, table, key, val */
-			lua_settable(state, -3);						/* table, key, table */
-		}
-		
-		/* add the card to the table */
-		lua_settable(state, -3);					/* table */
+			card[i] = Py_BuildValue("(ii)", trick->card[i]->suit, trick->card[i]->rank);
 	}
-
-	/* assign the table to the variable "trick" */
-	lua_setglobal(state, "trick");								/* <empty> */
+	
+	result = PyObject_CallMethod(object, "set", "(OOOO)iii", card[NORTH], card[EAST], card[SOUTH], card[WEST], trick->trump, trick->num_played, trick->first_played);
+	
+	Py_DECREF(result);
+	for (i = 0; i < 4; i++)
+		Py_DECREF(card[i]);
 }
 
 /* return the ID for a card */
