@@ -27,20 +27,23 @@
 typedef struct _Player
 {
 	gint 	  	score_total;
-	gint 	  	score_round;
-	gint		point_cards;
-	lua_State 	*lua_state;
+	gchar		*name;
 	CardsHand 	*hand;
+	GList		*cards_taken;
 	PangoLayout *layout;
+	PyObject	*ai;
+	Trick 		*trick;
 } Player;
 
 /* functions for the player */
-Player* player_new (gint direction, gint orientation, GString *script, GtkWidget *widget);
-void 	player_play (Player *player, Trick *trick);
-void 	player_select_cards (Player *player);
-void	player_trick_end(Player *player, Trick *trick);
+Player* player_new (gint direction, gboolean show_faces, gchar *script, Trick *trick, GtkWidget *widget);
+Card* 	player_play (Player *player);
+GList* 	player_select_cards (Player *player);
+void	player_trick_end(Player *player);
 void	player_round_end(Player *player);
-void 	player_play_card (Player *player, Card *card, Trick *trick);
+void 	player_play_card (Player *player, Card *card);
+void	player_mark_selected(Player *player, GList *list);
+void	player_take_trick(Player *player, Trick *trick);
 void 	player_free (Player *player);
 
 #endif
